![](https://ios.bytedance.net/wlapi/tosDownload/iosbinary/general/BDAnalyze.jpeg)
# BDAnalyze

BDAnalyze是一款用于分析iOS代码，快速解析iOS代码依赖关系的工具，同时可修复iOS代码的规范性引用问题

## Installation

Add this line to your application's Gemfile:

```ruby
gem 'BDAnalyze'
```

And then execute:

    $ bundle

Or install it yourself as:

    $ gem install BDAnalyze
    



## Feature
- [x] **[一键沉库，分析出不同层次的依赖关系](#Case1)**
- [x] **[源码级别分析代码之间的引用关系](#Case2)**
- [x] **[修复Pods之间不规范的引用方式](#Case3)**
- [x] **[分析组件的实际依赖与隐式依赖](#Case2)**

## Example
![](https://ios.bytedance.net/wlapi/tosDownload/iosbinary/general/BDAnalyze.gif)

#### <a id="Case1"></a>[1] 一键沉库，分析出不同层次的依赖关系


在大型项目的组件化过程中，沉库操作为最耗时也是最重要的步骤。

对于沉库，一般的操作步骤是先梳理出目标文件集合的所有依赖关系**「包含上层业务依赖、关联的Pod组件依赖以及若干层二级业务依赖和组件依赖」**，然后暴力沉库，直至编译通过。

使用BDAnalyze可以非常方便的梳理出给定目标文件的所有依赖关系「直接业务以来、关联组件依赖、二级以及若干级依赖」

假设我们要将头条项目中的 **tt_app_ios/Article/Model**文件进行沉库.需要准备的条件有：

```
1.一份编译好的头条 源码 工程「i.e.为了测试准确性以及完整性，需要用到所有源码信息，因此需要使用一份源码编译的头条工程,这里一定要注意需要使用全源码编译，因为二进制的Pods没有hmap信息」
2.一个用来存放解析结果「解析结果为：各种文件」的绝对路径。e.g.---- /Users/bob/des
3.头条项目工程在本机的绝对路径 e.g ---- /Users/bob/Git/work/tt_app_ios/Article/Article.xcodeproj
4.要分析的目标路径 e.g. ---- /Users/bob/Git/work/tt_app_ios/Article/Model「由于头条项目中使用了*.pch文件，因此解析时也要将pch文件放到目标文件下」
5.头条项目主工程的hmap文件 e.g. ---- /Users/bob/Library/Developer/Xcode/DerivedData/Article-gynljfcyrfnppnewlnrdnebqxclg/Build/Intermediates.noindex/Article.build/Debug-iphonesimulator/newsbeta.build/newsbeta.hmap
6.头条项目Pods工程的hmap文件[任意Pods的中的hmap文件即可] e.g. ---- /Users/bob/Library/Developer/Xcode/DerivedData/TTWenda-gynljfcyrfnppnewlnrdnebqxclg/Build/Intermediates.noindex/Pods.build/Debug-iphonesimulator/TTAccountLogin.build/TTAccountLogin.hmap「注意这里需要确定你编译的是真机还是模拟器，如果你选择的是真机但是实际用的模拟器编译，会导致Pods信息不全」
```
**其中第5条和第6条可以通过下面指令获取「以头条工程为例」**

```ruby
BDAnalyze --searchhmap Article

 ✅  👉  /Users/bob/Library/Developer/Xcode/DerivedData/Article-cghgvmicwdcbxwfdfbahfewrpppf/Build/Intermediates.noindex/Article.build/Debug-iphonesimulator/Explore.build/Explore.hmap
 ✅  👉  /Users/bob/Library/Developer/Xcode/DerivedData/Article-cghgvmicwdcbxwfdfbahfewrpppf/Build/Intermediates.noindex/Pods.build/Debug-iphonesimulator/AWEVideoPlayer.build/AWEVideoPlayer.hmap
 
```
具备上述条件后，可以终端输入以下指令

**Note：**

```
1.默认分析是不解析Pods的.m中的二级依赖,只分析依赖的哪些Pods，如需分析Pods中.m文件的二级依赖，请再添加--analyzePods指令
2.如果需要分析每一层解析的显式Pods与隐式Pods依赖「一般用于Development Pods的二次沉库」，则需要添加 --parseEachPods 指令
3.如果需要每层都生成Podfile文件的话，则需要添加 --podfileLock path[path代表你要分析的主工程的Podfile.lock的绝对路径]
4.如果需要分析不准确的情况，先确定是不是用的全源码工程，因为二进制工程中没有信息可提取
```

```ruby
BDAnalyze --s /Users/bob/des 
          --proj /Users/bob/Git/work/tt_app_ios/Article/Article.xcodeproj
          --tpath /Users/bob/Git/工作/tt_app_ios/Article/Model 
          --mHmap /Users/bob/Library/Developer/Xcode/DerivedData/Article-cghgvmicwdcbxwfdfbahfewrpppf/Build/Intermediates.noindex/Article.build/Debug-iphonesimulator/Explore.build/Explore.hmap 
          --pHmap /Users/bob/Library/Developer/Xcode/DerivedData/Article-cghgvmicwdcbxwfdfbahfewrpppf/Build/Intermediates.noindex/Pods.build/Debug-iphonesimulator/AWEVideoPlayer.build/AWEVideoPlayer.hmap
```

**如果是想要将Development Pods中的Pods进行独立拆解的话，请运行这段代码**

```ruby
BDAnalyze --s /Users/bob/des 
          --proj /Users/bob/Git/work/tt_app_ios/Article/Article.xcodeproj
          --tpath /Users/bob/Git/工作/tt_app_ios/Article/Model 
          --mHmap /Users/bob/Library/Developer/Xcode/DerivedData/Article-cghgvmicwdcbxwfdfbahfewrpppf/Build/Intermediates.noindex/Article.build/Debug-iphonesimulator/Explore.build/Explore.hmap 
          --pHmap /Users/bob/Library/Developer/Xcode/DerivedData/Article-cghgvmicwdcbxwfdfbahfewrpppf/Build/Intermediates.noindex/Pods.build/Debug-iphonesimulator/AWEVideoPlayer.build/AWEVideoPlayer.hmap
          --analyzePods
          --parseEachPods
          --podfileLock /Users/bob/Git/工作/tt_app_ios/Article/Podfile.lock
          
```

等待几秒后会输出如下：

```ruby
  ☠️ RecommendUser.h 没有被使用,不在工程目录 所属文件:/Users/bob/Git/工作/tt_app_ios/Article/Model/DataModel/RecommendUser.m

  ☠️ PadActionButton.h 没有被使用,不在工程目录 所属文件:/Users/bob/Git/工作/tt_app_ios/Article/NewsBase/view/ActionButton.m

  ☠️ UIColorAdditions.h 没有被使用,不在工程目录 所属文件:/Users/bob/Git/工作/tt_app_ios/Article/NewsBase/view/ActionButton.m

  💯 解析完毕，请到 👉 /Users/bob/des 👈 查看解析结果
```
**其中类似警告的信息表示，代码中通过import xxx方式引入，但是实际编译时没有用到**

然后根据提示到**/Users/bob/des**文件下，查看解析结果「实际解析关联太多，只截取部分」：

```ruby
.
├── allDenpendencyPod.txt
├── 第10层
│   ├── HTSVideoMusicClass.h
│   ├── HTSVideoMusicDetail.h
│   ├── HTSVideoMusicList.h
│   ├── containedPods.txt
│   ├── user_header_path.txt
│   └── Pods
│       └── AWEVideoPlayer
│           └── AWEVideoCachePlayTask.h
├── 第11层
│   ├── CADisplayLink+HTSBlock.h
│   ├── CADisplayLink+HTSBlock.m
│   ├── HTSVideoMusicCommon.h
│   ├── user_header_path.txt
│   └── containedPods.txt
├── 第1层
│   ├── Article+TTADComputedProperties.h
│   ├── Article+TTADComputedProperties.m
│   ├── ArticleMomentManager.h
│   ├── user_header_path.txt
│   └── containedPods.txt
├── 第2层
│   ├── ArticleAddressManager.h
│   ├── ArticleAddressManager.m
│   ├── user_header_path.txt
│   └── containedPods.txt
├── 第3层
│   ├── ArticleAPNsManager.h
│   ├── ArticleAPNsManager.m
│   ├── user_header_path.txt
│   └── containedPods.txt
├── 第4层
│   ├── APNsManager.h
│   ├── APNsManager.m
│   ├── UIViewController+BDTAccountModalPresentor.h
│   ├── UIViewController+BDTAccountModalPresentor.m
│   ├── user_header_path.txt
│   └── containedPods.txt
├── 第5层
│   ├── ActionButton.h
│   ├── ActionButton.m
│   ├── UIViewController+TTVAdditions.m
│   ├── user_header_path.txt
│   └── containedPods.txt
├── 第6层
│   ├── ArticleAvatarView.h
│   ├── ArticleAvatarView.m
│   ├── containedPods.txt
│   ├── user_header_path.txt
│   └── toutiao.hpp
├── 第7层
│   ├── AccountButton.h
│   ├── AccountButton.m
│   ├── TTVideoRecommendView.m
│   ├── user_header_path.txt
│   └── containedPods.txt
├── 第8层
│   ├── ArticleMomentListCellCommentItem.h
│   ├── ArticleMomentListCellCommentItem.m
│   ├── user_header_path.txt
│   └── containedPods.txt
└── 第9层
    ├── HTSVideoMusicInfoDataManager.h
    ├── HTSVideoMusicInfoDataManager.m
    ├── user_header_path.txt
    └── containedPods.txt

11 directories, 2579 files
```

解析结果按照 **依赖层级** 进行分布，上图即说明 **/Users/bob/Git/工作/tt_app_ios/Article/Model** 文件集合共有11层依赖。其中每层依赖的文件夹下表示当前依赖层级的所有文件，其中对于Pods的依赖存放在每个层级的containedPods.txt文件中。在最外层的allDenpendencyPod.txt文件表示所依赖的所有Pods统计.user_header_path.txt表示依赖文件的相对于--proj 的路径
#### <a id="Case2"></a>[2] 分析单个组件的实际依赖情况
在沉库过程中，还有一个重要操作就是梳理已经下沉的组件的实际依赖关系，这样可以提供最小颗粒度的组件给使用方，同时有序的依赖关系会带来解析速度的提升以及维护成本的下降

以分析头条项目中 BDAccount 组件为例，需要准备的条件有

```ruby
1.一份编译好的头条源码工程「i.e.为了测试准确性以及完整性，需要用到所有源码信息，因此需要使用一份源码编译的头条工程」
2.一个用来存放解析结果「解析结果为：各种文件」的绝对路径。e.g.---- /Users/bob/des
3.头条项目在本机的绝对路径 e.g ---- /Users/bob/Git/work/tt_app_ios
4.要分析的目标路径 e.g. ---- /Users/bob/Git/工作/tt_app_ios/Article/Pods/BDAccount」
5.头条项目主工程的hmap文件 e.g. ---- /Users/bob/Library/Developer/Xcode/DerivedData/Article-gynljfcyrfnppnewlnrdnebqxclg/Build/Intermediates.noindex/Article.build/Debug-iphonesimulator/newsbeta.build/newsbeta.hmap
6.头条项目Pods工程的hmap文件[任意Pods的中的hmap文件即可] e.g. ---- /Users/bob/Library/Developer/Xcode/DerivedData/TTWenda-gynljfcyrfnppnewlnrdnebqxclg/Build/Intermediates.noindex/Pods.build/Debug-iphonesimulator/TTAccountLogin.build/TTAccountLogin.hmap
7.打开--analyzePods指令，因为需要分析组件的实现源码依赖情况
7.打开--parseEachPods指令，可以分析组件的实际依赖与隐藏依赖
```
然后使用如下指令进行分析

```ruby
BDAnalyze --s /Users/bob/des 
          --proj /Users/bob/Git/work/tt_app_ios/Article/Article.xcodeproj
          --tpath /Users/bob/Git/工作/tt_app_ios/Article/Pods/BDAccount 
          --mHmap /Users/bob/Library/Developer/Xcode/DerivedData/Article-cghgvmicwdcbxwfdfbahfewrpppf/Build/Intermediates.noindex/Article.build/Debug-iphonesimulator/Explore.build/Explore.hmap 
          --pHmap /Users/bob/Library/Developer/Xcode/DerivedData/Article-cghgvmicwdcbxwfdfbahfewrpppf/Build/Intermediates.noindex/Pods.build/Debug-iphonesimulator/AWEVideoPlayer.build/AWEVideoPlayer.hmap
          --analyzePods
          --parseEachPods
          --podfileLock /Users/bob/Git/工作/tt_app_ios/Article/Podfile.lock
```

等待几秒后会输出如下：

```ruby
  ☠️ BDAccountPlatformManager.h 没有被使用,不在工程目录 所属文件:/Users/bob/Git/工作/tt_app_ios/Article/Pods/BDAccount/BDAccount/BDAccountSDK.h

  ☠️ BDAccountSinaWeiboRequest.h 没有被使用,不在工程目录 所属文件:/Users/bob/Git/工作/tt_app_ios/Article/Pods/BDAccount/BDAccount/BDAccountSDK.h

  ☠️ BDAccountFacebookRequest.h 没有被使用,不在工程目录 所属文件:/Users/bob/Git/工作/tt_app_ios/Article/Pods/BDAccount/BDAccount/BDAccountSDK.h

  ☠️ BDAccountGoogleRequest.h 没有被使用,不在工程目录 所属文件:/Users/bob/Git/工作/tt_app_ios/Article/Pods/BDAccount/BDAccount/BDAccountSDK.h

  ☠️ BDAccountTwitterRequest.h 没有被使用,不在工程目录 所属文件:/Users/bob/Git/工作/tt_app_ios/Article/Pods/BDAccount/BDAccount/BDAccountSDK.h

  ☠️ zlib.h 没有被使用,不在工程目录 所属文件:/Users/bob/Git/工作/tt_app_ios/Article/Pods/Godzippa/Godzippa/NSData+Godzippa.h

  ☠️ BDSDKApiObject+CompanyProduct 没有被使用,不在工程目录 所属文件:/Users/bob/Git/工作/tt_app_ios/Article/Pods/Bytedancebase/Bytedancebase/BDOpenSDK/Privates/BDSDKUserPromotionInteractor.m

  ☠️ errno.h 没有被使用,不在工程目录 所属文件:/Users/bob/Git/工作/tt_app_ios/Article/Pods/TTNetworkManager/Pod/Classes/TTNetworkBase/NetworkUtil/TTNetworkHTTPErrorCodeMapper.m

  💯 解析完毕，请到 👉  /Users/bob/des  👈 查看解析结果
```

**其中类似警告的信息表示，代码中通过import xxx方式引入，但是实际编译时没有用到**

根据提示在 **/Users/bob/des** 下，找到解析结果如下

```
.
├── Podfile
├── allDenpendencyPod.txt
├── 第1层
│   ├── Podfile
│   ├── Pods
│   │   ├── BDAccount
│   │   │   └── BDAccountFlowscene.h
│   │   ├── Bytedancebase
│   │   │   ├── BDSDKApi+CompanyProduct.h
│   │   │   ├── BDSDKApi.h
│   │   │   └── BDSDKApi.m
│   │   ├── TTMonitor
│   │   │   ├── TTMonitor.h
│   │   │   └── TTMonitor.m
│   │   ├── TTNetworkManager
│   │   │   ├── TTHTTPRequestSerializerBase.h
│   │   ├── TTThirdPartySDKs
│   │   │   ├── QQApiInterface.h
│   │   │   └── WXApi.h
│   │   ├── TTTracker
│   │   │   ├── TTTracker.h
│   │   │   └── TTTracker.m
│   │   └── YYModel
│   │       └── YYModel.h
│   └── containedPods.txt
├── 第2层
│   ├── Podfile
│   ├── Pods
│   │   ├── Bytedancebase
│   │   │   ├── BDSDKApi+Privates.h
│   │   │   └── BDSDKUtils.m
│   │   ├── TTInstallService
│   │   │   └── TTInstallSandBoxHelper.m
│   │   ├── TTMonitor
│   │   │   ├── TTDebugRealMonitorManager.h
│   │   │   └── TTSystemMonitorManager.m
│   │   ├── TTNetworkManager
│   │   │   ├── TTHTTPJSONResponseSerializerBaseAFNetworking.h
│   │   │   └── TTNetworkManagerMonitorNotifier.m
│   │   ├── TTThirdPartySDKs
│   │   │   ├── QQApiInterfaceObject.h
│   │   │   ├── TencentMessageObject.h
│   │   │   ├── TencentOAuthObject.h
│   │   │   ├── WXApiObject.h
│   │   │   └── sdkdef.h
│   │   ├── TTTracker
│   │   │   ├── TTLogServer.h
│   │   │   └── TTTrackerUtil.m
│   │   └── YYModel
│   │       ├── NSObject+YYModel.h
│   │       └── YYClassInfo.m
│   └── containedPods.txt
├── 第3层
│   ├── Podfile
│   ├── Pods
│   │   ├── AFNetworking
│   │   │   └── AFNetworking.h
│   │   ├── AFgzipRequestSerializer
│   │   │   ├── AFgzipRequestSerializer.h
│   │   │   └── AFgzipRequestSerializer.m
│   │   ├── Bytedancebase
│   │   │   ├── BDSDKSessionManager.h
│   │   │   └── BDSDKUserPromotionInteractor.m
│   │   ├── FMDB
│   │   │   └── FMDB.h
│   │   ├── Godzippa
│   │   │   ├── NSData+Godzippa.h
│   │   │   └── NSData+Godzippa.m
│   │   ├── TTInstallService
│   │   │   ├── NSData+TTInstallAdditions.h
│   │   │   └── UIDevice+TTInstallAdditions.m
│   │   ├── TTMonitor
│   │   │   ├── TTAppPerformanceMonitorRecorder.h
│   │   │   └── TTWatchdogMonitorRecorder.mm
│   │   ├── TTNetworkManager
│   │   │   ├── TTDefaultHTTPRequestSerializer.h
│   │   │   ├── TTResponseModelProtocol.h
│   │   │   └── encrypt.h
│   │   ├── TTPushManager
│   │   │   └── message.h
│   │   └── TTTracker
│   │       └── TTTrackerStorageManager.m
│   └── containedPods.txt
├── 第4层
│   ├── Podfile
│   ├── Pods
│   │   ├── AFNetworking
│   │   │   ├── AFHTTPSessionManager.h
│   │   │   └── AFURLResponseSerialization.m
│   │   ├── FMDB
│   │   │   └── FMResultSet.m
│   │   ├── TTInstallService
│   │   │   └── TTInstallOpenUDID.h
│   │   ├── TTMonitor
│   │   │   └── TTWatchdogMonitor.m
│   │   ├── TTNetworkManager
│   │   │   └── TTNetworkReachabilityManager.m
│   │   └── TTReachability
│   │       ├── TTReachability.h
│   │       └── TTReachability.m
│   └── containedPods.txt
├── 第5层
│   ├── Podfile
│   ├── Pods
│   │   ├── AFNetworking
│   │   │   └── AFURLSessionManager.m
│   │   └── TTMonitor
│   │       ├── BSBacktraceLogger.h
│   │       └── BSBacktraceLogger.m
│   └── containedPods.txt
└── 第6层
    ├── Podfile
    ├── Pods
    │   └── AFNetworking
    │       ├── AFSecurityPolicy.h
    │       └── AFSecurityPolicy.m
    └── containedPods.txt

45 directories, 281 files
```
allDenpendencyPod.txt代表目标文件所依赖的**所有Pods统计**;
各层级表示相应的依赖层级依赖，比如第一层表示目标文件直接依赖的情况;
每一层的Pods文件夹表示该层依赖的Pods文件中的集合「只有直接依赖的文件，并不是每个Pod的所有依赖」;
每一层中的containedPods.txt表示该层的依赖解析情况，比如BDAccount的解析情况：

```
BDAccount 0.3.18-alpha.1
TTNetworkManager 2.2.8.38-rc.1
TTMonitor 0.7.9.42.3
YYModel 1.0.4
Bytedancebase 0.2.3
TTTracker 1.0.2
TTThirdPartySDKs 0.1.1

以下为隐式依赖：
TTInstallService 1.0.0.6
```

#### <a id="Case3"></a>[3] 修复Pods之间不规范的引用方式

由于历史遗留问题，在项目中还存在很多不规范的引用方式，比如

```objc
#import "AFNetworking.h"
#import <AFNetworking.h>
```
这种代码对于Framework方式或是swift混编的工程，往往会编译不过，但是一个一个去校验的工程量又非常大，因此这里提供了一种**一键修复**的方式。
以头条项目为例[其他壳工程类似]，需要准备的条件有

```
1.一份编译好的头条 源码 工程「i.e.为了测试准确性以及完整性，需要用到所有源码信息，因此需要使用一份源码编译的头条工程,这里一定要注意需要使用全源码编译，因为二进制的Pods没有hmap信息」
2.要修复的目标路径 e.g. ---- /Users/bob/Git/work/tt_app_ios/
3.头条项目Pods工程的hmap文件[任意Pods的中的hmap文件即可] e.g. ---- /Users/bob/Library/Developer/Xcode/DerivedData/TTWenda-gynljfcyrfnppnewlnrdnebqxclg/Build/Intermediates.noindex/Pods.build/Debug-iphonesimulator/TTAccountLogin.build/TTAccountLogin.hmap「注意这里需要确定你编译的是真机还是模拟器，如果你选择的是真机但是实际用的模拟器编译，会导致Pods信息不全」
```

只需要执行下面的指令即可

```ruby
BDAnalyze --repairAll /Users/bob/Git/工作/tt_app_ios
          --pHmap /Users/bob/Library/Developer/Xcode/DerivedData/Article-cghgvmicwdcbxwfdfbahfewrpppf/Build/Intermediates.noindex/Pods.build/Debug-iphonesimulator/AWEVideoPlayer.build/AWEVideoPlayer.hmap
```
执行完毕后，会生成修改的文件信息

```
  ✅  TTHttpRequest.h is repair

  ✅  TTLGiftSyncDefines.h is repair

  ✅  SSViewControllerBase.h is repair

  ✅  SSViewControllerBase.h is repair

  ✅  SSViewControllerBase.h is repair

  ✅  NSObject+MTLComparisonAdditions.h is repair

  💯 修复完毕，请到 👉  /Users/bob/Git/工作/tt_app_ios  👈 查看修复结果

  🏆 引用修复报告  👉  /Users/bob/Git/工作/tt_app_ios/Repair.json  👈
```
## License

The gem is available as open source under the terms of the [MIT License](https://opensource.org/licenses/MIT).

## Code of Conduct

Everyone interacting in the BDAnalyze project’s codebases, issue trackers, chat rooms and mailing lists is expected to follow the [code of conduct](https://github.com/[USERNAME]/BDAnalyze/blob/master/CODE_OF_CONDUCT.md). 

